/*
 * This file is part of vomitpage.
 * Copyright (C) 2012  Guillaume Quintin.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

/*******************************************************
          INDENTATION SHIT FOR GROFF
*******************************************************/

void man_indent(void) {
  int tmp;
  tmp = man_next_indent - man_last_indent;
  if ( tmp != 0 ) fprintf(stdout,".RS %d\n",tmp);
  man_last_indent = man_next_indent;
}

/*******************************************************
              UNORDERED LISTS
*******************************************************/

void list_begin(void) {
  switch(of) {
    case MAN:
      man_indent();
      fprintf(stdout,".IP \"\\-\" %d\n",MAN_TABSTOP);
      man_next_indent += MAN_TABSTOP;
      break;
    case HTML:
      fputs("<ul>\n<li><p>",stdout);
      break;
    case LATEX:
      fputs("\\begin{itemize}\n\\item ",stdout);
      break;
    default:
      break;
  }
}

void list_before_nested(void) {
  switch(of) {
    case MAN:
      fputc('\n',stdout);
      break;
    case HTML:
      fputs("</p>\n",stdout);
      break;
    case LATEX:
      fputc('\n',stdout);
      break;
    default:
      break;
  }
}

void list_append_item(void) {
  switch(of) {
    case MAN:
      man_next_indent -= MAN_TABSTOP;
      man_indent();
      man_next_indent += MAN_TABSTOP;
      fputs(".IP \"\\-\" 2\n",stdout);
      break;
    case HTML:
      fputs("</li>\n<li><p>",stdout);
      break;
    case LATEX:
      fputs("\\item ",stdout);
      break;
    default:
      break;
  }
}

void list_end(void) {
  switch(of) {
    case MAN:
      man_next_indent -= MAN_TABSTOP;
      break;
    case HTML:
      fputs("</li>\n</ul>\n",stdout);
      break;
    case LATEX:
      fputs("\\end{itemize}\n",stdout);
      break;
    default:
      break;
  }
}

/*******************************************************
               INDENTED PARAGRAPHS
*******************************************************/

void ip_begin(void) {
  switch(of) {
    case MAN:
      man_indent();
      fputs(".IP \"",stdout);
      man_next_indent += 2 * MAN_TABSTOP;
      break;
    case HTML:
      fputs("<dl>\n<dt>",stdout);
      break;
    case LATEX:
      fputs("\\begin{description}\n"
            "\\item[",stdout);
      break;
    default:
      break;
  }
}

void ip_end_item_begin_para(void) {
  switch(of) {
    case MAN:
      fputs("\" 4\n",stdout);
      break;
    case HTML:
      fputs("</dt>\n<dd><p>",stdout);
      break;
    case LATEX:
      fputs("] ",stdout);
      break;
    default:
      break;
  }
}

void ip_before_nested(void) {
  switch(of) {
    case MAN:
      fputc('\n',stdout);
      break;
    case HTML:
      fputs("</p>\n",stdout);
      break;
    case LATEX:
      fputc('\n',stdout);
      break;
    default:
      break;
  }
}

void ip_append_item(void) {
  switch(of) {
    case MAN:
      man_next_indent -= 2 * MAN_TABSTOP;
      man_indent();
      man_next_indent += 2 * MAN_TABSTOP;
      fputs(".IP \"",stdout);
      break;
    case HTML:
      fputs("</dd>\n<dt>",stdout);
      break;
    case LATEX:
      fputs("\\item[",stdout);
      break;
    default:
      break;
  }
}

void ip_end(void) {
  switch(of) {
    case MAN:
      man_next_indent -= 2 * MAN_TABSTOP;
      break;
    case HTML:
      fputs("</dd>\n</dl>\n",stdout);
      break;
    case LATEX:
      fputs("\\end{description}\n",stdout);
      break;
    default:
      break;
  }
}

/*******************************************************
                    INLINE CODE
*******************************************************/

void inline_code_begin(void) {
  switch(of) {
    case MAN:
      fputs("\\fB",stdout);
      break;
    case HTML:
      fputs("<code>",stdout);
      break;
    case LATEX:
      fputs("\\texttt{",stdout);
      break;
    default:
      break;
  }
}

void inline_code_end(void) {
  switch(of) {
    case MAN:
      fputs("\\fR",stdout);
      break;
    case HTML:
      fputs("</code>",stdout);
      break;
    case LATEX:
      fputs("}",stdout);
      break;
    default:
      break;
  }
}

/*******************************************************
                INLINE EMPHASIZED
*******************************************************/

void inline_em_begin(void) {
  switch(of) {
    case MAN:
      fputs("\\fI",stdout);
      break;
    case HTML:
      fputs("<em>",stdout);
      break;
    case LATEX:
      fputs("\\emph{",stdout);
      break;
    default:
      break;
  }
}

void inline_em_end(void) {
  switch(of) {
    case MAN:
      fputs("\\fR",stdout);
      break;
    case HTML:
      fputs("</em>",stdout);
      break;
    case LATEX:
      fputs("}",stdout);
      break;
    default:
      break;
  }
}

/*******************************************************
                  PARAGRAPH
*******************************************************/

void para_begin(void) {
  switch(of) {
    case MAN:
      man_indent();
      fputs(".P\n",stdout);
      break;
    case HTML:
      fputs("<p>",stdout);
      break;
    case LATEX:
      break;
    default:
      break;
  }
}

void para_end(void) {
  switch(of) {
    case MAN:
      fputc('\n',stdout);
      break;
    case HTML:
      fputs("</p>\n",stdout);
      break;
    case LATEX:
      fputs("\n",stdout);
      break;
    default:
      break;
  }
}

/*******************************************************
                SUBSECTION
*******************************************************/

void subsection_begin(void) {
  switch(of) {
    case MAN:
      man_next_indent = 0;
      man_indent();
      fputs(".SS ",stdout);
      break;
    case HTML:
      fputs("<h3>",stdout);
      break;
    case LATEX:
      fputs("\\subsection{",stdout);
      break;
    default:
      break;
  }
}

void subsection_end(void) {
  switch(of) {
    case MAN:
      fputs("\n",stdout);
      break;
    case HTML:
      fputs("</h3>\n",stdout);
      break;
    case LATEX:
      fputs("}\n",stdout);
      break;
    default:
      break;
  }
}

/*******************************************************
                    SECTION
*******************************************************/

void section_begin(void) {
  switch(of) {
    case MAN:
      man_next_indent = 0;
      man_indent();
      fputs(".SH ",stdout);
      break;
    case HTML:
      fputs("<h2>",stdout);
      break;
    case LATEX:
      fputs("\\section{",stdout);
      break;
    default:
      break;
  }
}

void section_end(void) {
  switch(of) {
    case MAN:
      fputs("\n",stdout);
      break;
    case HTML:
      fputs("</h2>\n",stdout);
      break;
    case LATEX:
      fputs("}\n",stdout);
      break;
    default:
      break;
  }
}

/*******************************************************
                  VERBATIM
*******************************************************/

void code_begin(void) {
  switch(of) {
    case MAN:
      man_next_indent += MAN_TABSTOP;
      man_indent();
      fputs(".P\n.nf\n",stdout);
      break;
    case HTML:
      fputs("<pre><code>",stdout);
      break;
    case LATEX:
      fputs("\\begin{verbatim}",stdout);
      break;
    default:
      break;
  }
}

void code_end(void) {
  switch(of) {
    case MAN:
      fputs("\n.fi\n",stdout);
      man_next_indent -= MAN_TABSTOP;
      break;
    case HTML:
      fputs("</code></pre>\n",stdout);
      break;
    case LATEX:
      fputs("\\end{verbatim}\n",stdout);
      break;
    default:
      break;
  }
}

/*******************************************************
               OUTPUT CHARACTER
*******************************************************/

void put_char(char c) {
  switch(of) {
    case MAN:
      switch(c) {
        case '\\':
          fputs("\\e",stdout);
          break;
        case '\'':
          fputs("\\(aq",stdout);
          break;
        case '`':
          fputs("\\(oq",stdout);
          break;
        case '"':
          fputs("\\(dq",stdout);
          break;
        case '-':
          fputs("\\-",stdout);
          break;
        default:
          fputc(c,stdout);
          break;
      }
      break;
    case HTML:
      switch(c) {
        case '<':
          fputs("&lt;",stdout);
          break;
        case '>':
          fputs("&gt;",stdout);
          break;
        case '&':
          fputs("&amp;",stdout);
          break;
        case '"':
          fputs("&quot;",stdout);
          break;
        case '\'':
          fputs("&apos;",stdout);
          break;
        default:
          fputc(c,stdout);
          break;
      }
      break;
    case LATEX:
      if ( blocks[curr_block] == BLOCK_CODE ) {
        fputc(c,stdout);
        break;
      }
      switch(c) {
        case '-':
          fputs("{-}",stdout);
          break;
        case '$':
          fputs("\\$",stdout);
          break;
        case '#':
          fputs("\\#",stdout);
          break;
        case '%':
          fputs("\\%%",stdout);
          break;
        case '^':
          fputs("\\^",stdout);
          break;
        case '&':
          fputs("\\&",stdout);
          break;
        case '_':
          fputs("\\_",stdout);
          break;
        case '~':
          fputs("\\~{}",stdout);
          break;
        case '\\':
          fputs("\\textbackslash{}",stdout);
          break;
        case '{':
          fputs("\\{",stdout);
          break;
        case '}':
          fputs("\\}",stdout);
          break;
        default:
          fputc(c,stdout);
          break;
      }
  }
}

