/*
 * This file is part of vomitpage.
 * Copyright (C) 2012  Guillaume Quintin.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

void enter_block(int type) {
  curr_block++;
  if ( curr_block >= MAX_DEPTH ) error("too deeply nested");
  blocks[curr_block] = type;
  switch(type) {
    case BLOCK_NORMAL:
      para_begin();
      break;
    case BLOCK_LIST:
      list_begin();
      break;
    case BLOCK_CODE:
      code_begin();
      break;
    case BLOCK_INDENT:
      ip_begin();
      break;
    default:
      break;
  }
}

void exit_block(int n) {
  int i;
  for( i = 0 ; i < n && curr_block >= 0 ; i++,curr_block-- ) {
    switch(blocks[curr_block]) {
      case BLOCK_NORMAL:
        para_end();
        break;
      case BLOCK_LIST:
        list_end();
        break;
      case BLOCK_CODE:
        code_end();
        break;
      case BLOCK_INDENT:
        ip_end();
        break;
      default:
        break;
    }
  }
}

void exit_all_blocks(void) {
  exit_block(curr_block + 1);
}

void compile_code(const char *end,int ign) {
  for( ; win[0] ; ) {
    if ( !cmp(end) ) return;
    if ( win[0] == '\\' && win[1] ) { /* escape character */
      if ( win[1] != '\n' ) put_char(win[1]);
      shift_for_blockcode(2,ign);
      continue;
    }
    put_char(win[0]);
    shift_for_blockcode(1,ign);
  }
}

void compile_inline(const char *end) {
  int em,code,em_lineno,code_lineno;
  em = 0;
  em_lineno = 0;
  code = 0;
  code_lineno = 0;
  for( ; win[0] ; ) {
    if ( !cmp(end) ) break;
    if ( !cmp("`") ) {
      code_lineno = get_real_lineno();
      code = !code;
      if ( code ) inline_code_begin(); else inline_code_end();
      shift(1);
      continue;
    }
    if ( !code && !cmp("*") ) {
      em_lineno = get_real_lineno();
      em = !em;
      if ( em ) inline_em_begin(); else inline_em_end();
      shift(1);
      continue;
    }
    if ( win[0] == '\\' && win[1] ) {
      if ( win[1] != '\n' ) put_char(win[1]);
      shift(2);
      continue;
    }
    put_char(win[0]);
    shift(1);
  }
  if ( em ) warning("emphasized end mark missing?"
                    " (began at line %d)",em_lineno);
  if ( code ) warning("inline code end mark missing??"
                      " (began at line %d)",code_lineno);
}

void skip_comment(void) {
  for( ; win[0] ; ) {
    if ( !cmp("##\n") ) return;
    shift(1);
  }
}

void compile(void) {
  for( ; win[0] ; ) {
    if ( win[0] != '\n' ) {
      int new_block;
      new_block = indent[0] / TABSTOP;
      if ( TABSTOP * new_block != indent[0] ) {
        warning("indentation seems incorrect");
        warning("it should be a multiple of %d",TABSTOP);
      }
      if ( new_block > curr_block + 1 ) {
        warning("indentation seems incorrect");
        warning("it should not be that big");
      }
      if ( new_block < curr_block ) {
        exit_block(curr_block - new_block);
      }
      if ( !cmp("- ") ) {
        if ( new_block > curr_block ) {
          enter_block(BLOCK_LIST);
        }
        else if ( blocks[curr_block] == BLOCK_LIST ) {
          list_append_item();
        }
        else {
          exit_block(1);
          enter_block(BLOCK_LIST);
        }
        shift(2);
        compile_inline("\n\n");
        shift(2);
        list_before_nested();
        continue;
      }
      if ( !cmp("> ") ) {
        if ( new_block > curr_block ) {
          enter_block(BLOCK_INDENT);
        }
        else if ( blocks[curr_block] == BLOCK_INDENT ) {
          ip_append_item();
        }
        else {
          exit_block(1);
          enter_block(BLOCK_INDENT);
        }
        shift(2);
        compile_inline("\n");
        ip_end_item_begin_para();
        shift(1);
        compile_inline("\n\n");
        shift(2);
        ip_before_nested();
        continue;
      }
      if ( !cmp("##\n") ) {
        shift(3);
        skip_comment();
        shift(3);
        continue;
      }
      if ( !cmp("===") ) {
        exit_all_blocks();
        shift(3);
        subsection_begin();
        compile_inline("===");
        subsection_end();
        shift(3);
        continue;
      }
      if ( !cmp("==") ) {
        exit_all_blocks();
        shift(2);
        section_begin();
        compile_inline("==");
        section_end();
        shift(2);
        continue;
      }
      if ( !cmp("``\n") ) {
        if ( new_block <= curr_block ) {
          exit_block(curr_block - new_block + 1);
        }
        enter_block(BLOCK_CODE);
        shift(3);
        compile_code("\n``",new_block * 2);
        shift(3);
        continue;
      }
      /* normal text paragraph here !!! */
      if ( new_block <= curr_block ) {
        exit_block(curr_block - new_block + 1);
      }
      enter_block(BLOCK_NORMAL);
      compile_inline("\n\n");
      shift(2);
      continue;
    }
    shift(1);
  }
  exit_all_blocks();
}

