/*
 * This file is part of vomitpage.
 * Copyright (C) 2012  Guillaume Quintin.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

void usage(void) {
  fprintf(stderr,"Usage: vomitpage [ OPTION ] [ FILE ]\n\n"
                 "OPTION can be one of the following:\n"
                 "  --man   Generate roff.\n"
                 "  --html  Generate HTML.\n"
                 "  --latex Generate LaTeX.\n\n"
                 "FILE can be a list of file.\n"
                 "If FILE is `-' then read standard input.\n");
  fflush(stderr);
}

void error_with_usage(int show_usage,const char *s,...) {
  va_list ap;
  fputs("vomitpage: ",stderr);
  va_start(ap,s);
  vfprintf(stderr,s,ap);
  fputc('\n',stderr);
  fflush(stderr);
  va_end(ap);
  if ( show_usage ) {
    fputc('\n',stderr);
    usage();
  }
  exit(-1);
}

int get_real_lineno(void) {
  int r,i;
  r = lineno;
  for( i = 1 ; i < WIN_LEN && win[i] ; i++ ) {
    if ( win[i] == '\n' ) r--;
  }
  return r;
}

void error(const char *s,...) {
  va_list ap;
  fprintf(stderr,"%s:%d: error: ",filena,get_real_lineno());
  va_start(ap,s);
  vfprintf(stderr,s,ap);
  fputc('\n',stderr);
  fflush(stderr);
  va_end(ap);
  exit(-1);
}

void warning(const char *s,...) {
  va_list ap;
  fprintf(stderr,"%s:%d: warning: ",filena,get_real_lineno());
  va_start(ap,s);
  vfprintf(stderr,s,ap);
  fputc('\n',stderr);
  fflush(stderr);
  va_end(ap);
}

