/*
 * This file is part of vomitpage.
 * Copyright (C) 2012  Guillaume Quintin.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#define _POSIX_C_SOURCE 1

#include <time.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <stdarg.h>

/* what is the output */
#define NONE          -1
#define MAN            0
#define HTML           1
#define LATEX          2

/* the current parapgraph */
#define BLOCK_NONE    -1
#define BLOCK_NORMAL   0
#define BLOCK_CODE     1
#define BLOCK_LIST     2
#define BLOCK_INDENT   3

/* other defines */
#define WIN_LEN        3
#define VERSION       "vomitpage version 0.7"
#define MAX_DEPTH      10
#define TABSTOP        2
#define MAN_TABSTOP    2

/* global variables */
char win[WIN_LEN];
int indent[WIN_LEN];
int blocks[MAX_DEPTH],curr_block;
char last_char;
FILE *in;
int of;
int lineno;
const char *filena;
int man_next_indent,man_last_indent;

#include "utils.c"
#include "read.c"
#include "backend.c"
#include "compile.c"

void init(FILE *f,const char *s) {
  lineno = 1;
  filena = s;
  curr_block = -1;
  last_char = '\n';
  in = f;
  man_last_indent = 0;
  man_next_indent = 0;
}

int main(int argc,char **argv) {
  int i,start;
  FILE *f;
  if ( argc == 2 && !strcmp(argv[1],"-v") ) {
    puts(VERSION);
    return 0;
  }
  of = NONE;
  if ( argc > 1 ) {
    if ( !strcmp(argv[1],"--man") ) of = MAN;
    if ( !strcmp(argv[1],"--html") ) of = HTML;
    if ( !strcmp(argv[1],"--latex") ) of = LATEX;
  }
  if ( of == NONE ) {
    of = MAN;
    start = 1;
  }
  else {
    start = 2;
  }
  if ( argc > start ) {
    for( i = start ; i < argc ; i++ ) {
      if ( !strcmp(argv[i],"-") ) {
        init(stdin,"stdin");
        shift(WIN_LEN);
        compile();
        fclose(in);
      }
      else {
        f = fopen(argv[i],"r");
        if ( !f ) {
          error_with_usage(0,"%s: cannot open file",argv[i]);
        }
        init(f,argv[i]);
        shift(WIN_LEN);
        compile();
        fclose(f);
      }
    }
  }
  else {
    init(stdin,"stdin");
    shift(WIN_LEN);
    compile();
    fclose(in);
  }
  fflush(stdout);
  return 0;
}

